/* ipuz-crossword.h
 *
 * Copyright 2022 Jonathan Blandford <jrb@gnome.org>
 *
 * This file is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 3 of the
 * License, or (at your option) any later version.
 *
 * This file is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * SPDX-License-Identifier: LGPL-3.0-or-later
 */

#pragma once

#include <glib-object.h>
#include <libipuz/ipuz-puzzle.h>
#include <libipuz/ipuz-board.h>
#include <libipuz/ipuz-clue.h>
#include <libipuz/ipuz-guesses.h>

G_BEGIN_DECLS


#define IPUZ_TYPE_CROSSWORD (ipuz_crossword_get_type ())
G_DECLARE_DERIVABLE_TYPE    (IPuzCrossword, ipuz_crossword, IPUZ, CROSSWORD, IPuzPuzzle);


typedef enum
{
  IPUZ_SYMMETRY_NONE,    /* @ Freeform puzzle @ */
  IPUZ_SYMMETRY_QUARTER, /* @ 90° Rotational Symmetry around the center. Only works on square boards. */
  IPUZ_SYMMETRY_HALF,    /* @ 180° Rotational Symmetry around the center*/
} IPuzSymmetry;

typedef enum
{
  IPUZ_CLUE_PLACEMENT_NULL,   /* @ Auto place clues @ */
  IPUZ_CLUE_PLACEMENT_BEFORE, /* @ Put clues before the puzzle @ */
  IPUZ_CLUE_PLACEMENT_AFTER,  /* @ Put clues after the puzzle @ */
  IPUZ_CLUE_PLACEMENT_BLOCKS, /* @ Put clues in blocks adjacent to entry @ */
} IPuzCluePlacement;


typedef void (*IPuzClueForeachFunc) (IPuzClueDirection  direction,
                                     IPuzClue          *clue,
                                     IPuzClueId         clue_id,
                                     gpointer           user_data);

typedef struct _IPuzCrosswordClass IPuzCrosswordClass;
struct _IPuzCrosswordClass
{
  IPuzPuzzleClass parent_class;
};


IPuzCrossword     *ipuz_crossword_new                    (void);
guint              ipuz_crossword_get_width              (IPuzCrossword       *self);
guint              ipuz_crossword_get_height             (IPuzCrossword       *self);
void               ipuz_crossword_set_size               (IPuzCrossword       *self,
                                                          gint                 width,
                                                          gint                 height);
IPuzBoard         *ipuz_crossword_get_board              (IPuzCrossword       *self);
gboolean           ipuz_crossword_set_guesses            (IPuzCrossword       *self,
                                                          IPuzGuesses         *guesses);
IPuzGuesses       *ipuz_crossword_get_guesses            (IPuzCrossword       *self);
IPuzCell          *ipuz_crossword_get_cell               (IPuzCrossword       *self,
                                                          IPuzCellCoord        coord);

/* Clue sets */
guint              ipuz_crossword_get_n_clue_sets        (IPuzCrossword       *self);
IPuzClueDirection  ipuz_crossword_clue_set_get_dir       (IPuzCrossword       *self,
                                                          guint                index);
const gchar       *ipuz_crossword_clue_set_get_label     (IPuzCrossword       *self,
                                                          IPuzClueDirection    direction);


/* Clues */
GArray            *ipuz_crossword_get_clues              (IPuzCrossword       *self,
                                                          IPuzClueDirection    direction);
guint              ipuz_crossword_get_n_clues            (IPuzCrossword       *self,
                                                          IPuzClueDirection    direction);
IPuzClue          *ipuz_crossword_get_clue_by_id         (IPuzCrossword       *self,
                                                          IPuzClueId           clue_id);
IPuzClueId         ipuz_crossword_get_clue_id            (IPuzCrossword       *self,
                                                          const IPuzClue      *clue);
gchar             *ipuz_crossword_get_clue_string_by_id  (IPuzCrossword       *self,
                                                          IPuzClueId           clue_id);
gchar             *ipuz_crossword_get_guess_string_by_id (IPuzCrossword       *self,
                                                          IPuzClueId           clue_id);
gboolean           ipuz_crossword_clue_guessed           (IPuzCrossword       *self,
                                                          IPuzClue            *clue,
                                                          gboolean            *correct);
guint              ipuz_crossword_find_clue              (IPuzCrossword       *self,
                                                          IPuzClue            *clue);
IPuzClue          *ipuz_crossword_find_clue_by_number    (IPuzCrossword       *self,
                                                          IPuzClueDirection    direction,
                                                          gint                 number);
IPuzClue          *ipuz_crossword_find_clue_by_label     (IPuzCrossword       *self,
                                                          IPuzClueDirection    direction,
                                                          const char          *label);
IPuzClue          *ipuz_crossword_find_clue_by_coord     (IPuzCrossword       *self,
                                                          IPuzClueDirection    direction,
                                                          IPuzCellCoord        coord);
void               ipuz_crossword_clues_foreach          (IPuzCrossword       *self,
                                                          IPuzClueForeachFunc  func,
                                                          gpointer             data);

/* Game state */
gboolean           ipuz_crossword_game_won               (IPuzCrossword       *self);

/* Mutators */    
void               ipuz_crossword_calculate_clues        (IPuzCrossword       *self);
void               ipuz_crossword_set_symmetry           (IPuzCrossword       *self,
                                                          IPuzSymmetry         symmetry);
IPuzSymmetry       ipuz_crossword_get_symmetry           (IPuzCrossword       *self);
void               ipuz_crossword_set_cell_type          (IPuzCrossword       *self,
                                                          IPuzCellCoord        coord,
                                                          IPuzCellCellType     cell_type);

/* Public functions */
void               ipuz_crossword_print                  (IPuzCrossword       *self);


G_END_DECLS
